﻿using Microsoft.Extensions.Logging;
using RouteGuideContracts.BindingModels;
using RouteGuideContracts.BusinessLogicContracts;
using RouteGuideContracts.SearchModels;
using RouteGuideDataModels.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using static System.Windows.Forms.VisualStyles.VisualStyleElement;

namespace RouteGuide
{
    public partial class FormRoute : Form
    {
        private readonly ILogger _logger;
        private readonly IRouteLogic _logic;
        private int? _id;
        private Dictionary<int, (IStopModel, int)> _RouteStops;
        public int Id { set { _id = value; } }
        public FormRoute(ILogger<FormRoute> logger, IRouteLogic logic)
        {
            InitializeComponent();
            _logger = logger;
            _logic = logic;
            _RouteStops = new Dictionary<int, (IStopModel, int)>();
        }

        private void FormRoute_Load(object sender, EventArgs e)
        {
            if (_id.HasValue)
            {
                try
                {
                    var view = _logic.ReadElement(new RouteSearchModel
                    {
                        Id = _id.Value
                    });
                    if (view != null)
                    {
                        textBoxName.Text = view.Name;
                        _RouteStops = view.RouteStops ?? new Dictionary<int, (IStopModel, int)>();
                        LoadData();
                        //foreach (var el in view.StopRoutes.Values)
                        //{
                        //	dataGridView.Rows.Add(new object[] {el.Item1.Name, el.Item2});
                        //}
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                }
            }
        }

        private void LoadData()
        {
            try
            {
                if (_RouteStops != null)
                {
                    dataGridView.Rows.Clear();
                    foreach (var pc in _RouteStops)
                    {
                        dataGridView.Rows.Add(new object[] { pc.Key, pc.Value.Item1.Name, pc.Value.Item2 });
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
               MessageBoxIcon.Error);
            }
        }

        private void ButtonAdd_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormRouteStop));
            if (service is FormRouteStop form)
            {
                if (form.ShowDialog() == DialogResult.OK)
                {
                    if (form.StopModel == null)
                    {
                        return;
                    }
                    if (_RouteStops.ContainsKey(form.Id))
                    {
                        _RouteStops[form.Id] = (form.StopModel, form.Number);
                    }
                    else
                    {
                        _RouteStops.Add(form.Id, (form.StopModel, form.Number));
                    }
                    LoadData();
                }
            }

        }

        private void ButtonRefresh_Click(object sender, EventArgs e)
        {
            if (dataGridView.SelectedRows.Count == 1)
            {
                var service = Program.ServiceProvider?.GetService(typeof(FormRouteStop));
                if (service is FormRouteStop form)
                {
                    int id = Convert.ToInt32(dataGridView.SelectedRows[0].Cells[0].Value);
                    form.Id = id;
                    form.Number = _RouteStops[id].Item2;
                    if (form.ShowDialog() == DialogResult.OK)
                    {
                        if (form.StopModel == null)
                        {
                            return;
                        }
                        _RouteStops[form.Id] = (form.StopModel, form.Number);
                        LoadData();
                    }
                }
            }

        }

        private void ButtonDelete_Click(object sender, EventArgs e)
        {
            if (dataGridView.SelectedRows.Count == 1)
            {
                if (MessageBox.Show("Удалить запись?", "Вопрос",
               MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    try
                    {
                        _RouteStops?.Remove(Convert.ToInt32(dataGridView.SelectedRows[0].Cells[0].Value));
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(ex.Message, "Ошибка",
                       MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                    LoadData();
                }
            }

        }

        private void ButtonUpdate_Click(object sender, EventArgs e)
        {
            LoadData();
        }

        private void ButtonSave_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxName.Text))
            {
                MessageBox.Show("Fill in Name", "Ошибка",
               MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (_RouteStops == null || _RouteStops.Count == 0)
            {
                MessageBox.Show("Fill in Stops", "Ошибка",
               MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            try
            {
                var model = new RouteBindingModel
                {
                    Id = _id ?? 0,
                    Name = textBoxName.Text,
                    RouteStops = _RouteStops
                };
                var operationResult = _id.HasValue ? _logic.Update(model) :
               _logic.Create(model);
                if (!operationResult)
                {
                    throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                }
                MessageBox.Show("Сохранение прошло успешно", "Сообщение",
                MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка сохранения драгоценности");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
